;--------------------------------------------------------------------------
;    video_mz800_prnc.s - generic utility for MZ-800
;
;    Copyright (c) 2012 by Michal Hucik <http://www.ordoz.com>
;
;    Thies program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    driver_FDC is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with driver_FDC; if not, write to the Free Software
;    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
;--------------------------------------------------------------------------

	.module video_mz800_prnc

	.globl	_CG1

	.area _CODE


; Pozice kurzoru
_cur::
	.db	0x00	; pos_x
	.db	0x00	; pos_y



;
; HL - pattern (8 bajtu)
; DE - VRAM addr
;
__prn_pattern::

	ld b,#0x08

    prnc_next_micro$:
        ld a,(hl)
        
    _fl_INVPRN::
        .db 0x00
    
        ld (de),a
        inc hl
        
        push hl
        ld hl, #0x0050
        add hl,de

	ex de,hl

        pop hl

        djnz prnc_next_micro$

	ret


; Verejna funkce prnc.
;
; prevezme parametr funkce ze zasobniku
_prnc::
	pop bc
	pop hl
	push hl
	push bc
	ld a,l			; tisknuty znak

; Privatni funkce _prnc.
;
; a - znak
;
; Spocita si podle cur.xy adresu VRAM
;
; 
__prnc::
	ex af,af
	exx

        ; spocitat adresu VRAM podle cur.pos_xy
	;

	; Pricitaci metoda:
	;	celk min = 59
	;			+ ( radek * 24 ) - 5
	;	3 radek = 126 T
	;	4 radek = 150 T
	;	5 radek = 174 T
	;	6 radek = 198 T
	;
;        ld hl, #0x8000					;10
;        ld bc, (#_cur)					;20
;        xor a						;4
;        or b						;4
;        jp z, 2$					;10
;        ld de,#0x0280		; 1 radek 8 * 80	;10
;        
;    1$:
;        add hl, de					;11
;        djnz 1$					;8/13
;                
;    2$:
;        add hl,bc					;11




	; Nasobici metoda
	;
	; ( 0x8000 + x + ( y * 0x0280 ) )
	;
	;	celk vzdy = 184 T 
;	ld bc, (#_cur)			;20

;	ld e,b		; y		;4
;	ld a,e				;4
;	rla				;4
;	sbc a				;4
;	ld d,a 				;4
;	ld l,e 				;4
;	ld h,d 				;4
;	add hl,hl			;11
;	add hl,hl			;11
;	add hl,de			;11
;	add hl,hl			;11
;	add hl,hl			;11
;	add hl,hl			;11
;	add hl,hl			;11
;	add hl,hl			;11
;	add hl,hl			;11
;	add hl,hl			;11
;	ex de,hl			;4
;	ld h,#0x80	; 0x8000	;7
;	ld l,c		; x		;4
;	add hl,de			;11


	; Tabulkova metoda
	; celkem = 99 T	
	ld bc, (#_cur)			;20
	call __TabVRAMAddr


	push hl				; vram adresa
	exx
	pop de
	ex af,af

; Interni fce __prnc_vram	
; a - znak
; de - vram
;
; Zvysuje pos.x!
;
__prnc_vram:
        bit 7,a
        ret nz		; znaky nad 128 nechceme
        sub #0x20
        ret c		; znaky pod 32 nechceme
        
	push de
	exx
	pop de

        ; spocitat adresu patternu v CG1
        ld c,a
        xor a
        sla c
        rla
        sla c
        rla
        sla c
        rla
        ld b,a
        
        ld hl, #_CG1
        add hl, bc		; v HL je adresa znaku v CG1

	call __prn_pattern

        ; zvysit cur.pos_x
        ld hl, #_cur
        inc (hl)

	exx

	ret

;
; BC - y,x <0-39; 0-79>
;
__TabVRAMAddr:
	ld e,b				;4
	ld d,#0x00			;7
	ld b,d				;4
	sla e				;8
	ld hl,#__vram_row_tab		;10
	add hl,de			;11

	ld e,(hl)			;7
	inc hl				;6
	ld h,(hl)			;7
	ld l,e				;4
	add hl,bc			;11
	ret

; Tabulka adres pro kurzorove radky 0 - 39
__vram_row_tab:
	.dw	0x8000
	.dw	0x8280
	.dw	0x8500
	.dw	0x8780
	.dw	0x8a00
	.dw	0x8c80
	.dw	0x8f00
	.dw	0x9180
	.dw	0x9400
	.dw	0x9680
	.dw	0x9900
	.dw	0x9b80
	.dw	0x9e00
	.dw	0xa080
	.dw	0xa300
	.dw	0xa580
	.dw	0xa800
	.dw	0xaa80
	.dw	0xad00
	.dw	0xaf80
	.dw	0xb200
	.dw	0xb480
	.dw	0xb700
	.dw	0xb980
	.dw	0xbc00
	.dw	0xbe80
	.dw	0xc100
	.dw	0xc380
	.dw	0xc600
	.dw	0xc880
	.dw	0xcb00
	.dw	0xcd80
	.dw	0xd000
	.dw	0xd280
	.dw	0xd500
	.dw	0xd780
	.dw	0xda00
	.dw	0xdc80
	.dw	0xdf00
	.dw	0xe180


;
; Print message.
;
_msg::
	; vram addr pocitame jen jednou
	ld bc, (#_cur)			;20
	call __TabVRAMAddr
	ex de, hl		; v DE je adresa ve VRAM

	pop bc
	pop hl
	push hl
	push bc

     1$:
        ld a, (hl)
        cp #0x00
        ret z
        call __prnc_vram
        inc hl
	inc de		; zvysujeme vram addr
        jp 1$
        
